@@ -20,7 +20,7 @@
 import sys
 import subprocess
 import json
-import socket as socks
+import socket
 import struct
 import threading
 import time
@@ -29,8 +29,8 @@ ModuleType = type(sys)
 
 
 __author__ = 'Jure Ziberna'
-__version__ = '0.6.1'
-__date__ = '2012-04-19'
+__version__ = '0.6.4'
+__date__ = '2012-04-28'
 __license__ = 'GNU GPL 3'
 
 
@@ -74,11 +74,7 @@ class MessageError(i3Exception):
     Raised when a message to i3 is unsuccessful.
     That is, when it contains 'success': false in its JSON formatted response.
     """
-    @classmethod
-    def parse(cls, response):
-        if response and 'error' in response:
-            return cls(response['error'])
-        return None
+    pass
 
 class ConnectionError(i3Exception):
     """
@@ -92,7 +88,7 @@ class ConnectionError(i3Exception):
 def parse_msg_type(msg_type):
     """
     Returns an i3-ipc code of the message type. Raises an exception if
-    given message type isn't available.
+    the given message type isn't available.
     """
     try:
         index = int(msg_type)
@@ -108,8 +104,8 @@ def parse_msg_type(msg_type):
 
 def parse_event_type(event_type):
     """
-    Return an i3-ipc string of the event_type. Raises an exception if
-    given event type isn't available.
+    Returns an i3-ipc string of the event_type. Raises an exception if
+    the given event type isn't available.
     """
     try:
         index = int(event_type)
@@ -128,7 +124,7 @@ class Socket(object):
     """
     Socket for communicating with the i3 window manager.
     Optional arguments:
-    - path of the i3 socket. Path is retrieved from the i3-wm itself via
+    - path of the i3 socket. Path is retrieved from i3-wm itself via
       "i3.get_socket_path()" if not provided.
     - timeout in seconds
     - chunk_size in bytes
@@ -161,7 +157,7 @@ class Socket(object):
         """
         Initializes the socket.
         """
-        self.socket = socks.socket(socks.AF_UNIX, socks.SOCK_STREAM)
+        self.socket = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
         self.socket.settimeout(self.timeout)
     
     def connect(self, path=None):
@@ -174,7 +170,7 @@ class Socket(object):
                 path = self.path
             try:
                 self.socket.connect(path)
-            except socks.error:
+            except socket.error:
                 raise ConnectionError(path)
     
     def get(self, msg_type, payload=''):
@@ -209,7 +205,7 @@ class Socket(object):
     def receive(self):
         """
         Tries to receive a data. Unpacks the received byte string if
-        successful. Returns the current buffer on failure.
+        successful. Returns None on failure.
         """
         try:
             data = self.socket.recv(self.chunk_size)
@@ -220,7 +216,7 @@ class Socket(object):
                 data += self.socket.recv(msg_length)
             data = self.buffer + data
             return self.unpack(data)
-        except socks.timeout:
+        except socket.timeout:
             return None
     
     def pack(self, msg_type, payload):
@@ -271,7 +267,7 @@ class Socket(object):
         try:
             self.get('command')
             return True
-        except socks.error:
+        except socket.error:
             return False
     
     def close(self):
@@ -302,7 +298,7 @@ class Subscription(threading.Thread):
                  data_socket=None):
         # Variable initialization
         if not callable(callback):
-            raise TypeError('callback must be callable')
+            raise TypeError('Callback must be callable')
         event_type = parse_event_type(event_type)
         self.callback = callback
         self.event_type = event_type
@@ -326,14 +322,14 @@ class Subscription(threading.Thread):
         """
         try:
             self.listen()
-        except socks.error:
+        except socket.error:
             self.close()
     
     def listen(self):
         """
         Runs a listener loop until self.subscribed is set to False.
         Calls the given callback method with data and the object itself.
-        If event matches given one, then matching data is retrieved.
+        If event matches the given one, then matching data is retrieved.
         Otherwise, the event itself is sent to the callback.
         In that case 'change' key contains the thing that was changed.
         """
@@ -363,9 +359,7 @@ class Subscription(threading.Thread):
 
 def __call_cmd__(cmd):
     """
-    Excepts command arguments.
-    Returns output (stdout or stderr).
-    Uses subprocess module for executing the command.
+    Returns output (stdout or stderr) of the given command args.
     """
     try:
         output = subprocess.check_output(cmd)
@@ -378,8 +372,9 @@ def __call_cmd__(cmd):
 __socket__ = None
 def default_socket(socket=None):
     """
-    Returns an already initialized socket. The socket was created with
-    default values.
+    Returns i3.Socket object, which was initiliazed once with default values
+    if no argument is given.
+    Otherwise sets the default socket to the given socket.
     """
     global __socket__
     if socket and isinstance(socket, Socket):
@@ -391,33 +386,31 @@ def default_socket(socket=None):
 
 def msg(type, message=''):
     """
-    Excepts a message type and a message itself.
-    Talks to the i3 via socket.
-    Returns the response from the socket.
+    Takes a message type and a message itself.
+    Talks to the i3 via socket and returns the response from the socket.
     """
     response = default_socket().get(type, message)
     return response
 
 
-def __function__(type, message=''):
+def __function__(type, message='', *args, **crit):
     """
-    Excepts a message type and message itself.
-    Returns a function, which excepts arguments and adds them to the
-    message string, calls i3.msg with the resulting arguments and returns a
-    response. If message type was command, the function returns success value.
+    Accepts a message type, a message. Takes optional args and keyword
+    args which are present in all future calls of the resulting function.
+    Returns a function, which takes arguments and container criteria.
+    If message type was 'command', the function returns success value.
     """
-    def function(*args, **criteria):
-        msg_full = ' '.join([message] + list(args))
+    def function(*args2, **crit2):
+        msg_full = ' '.join([message] + list(args)  + list(args2))
+        criteria = dict(crit)
+        criteria.update(crit2)
         if criteria:
             msg_full = '%s %s' % (container(**criteria), msg_full)
         response = msg(type, msg_full)
-        error = MessageError.parse(response)
-        if error:
-            raise error
-        elif parse_msg_type(type) == 0:  # if it's a command message, then
-            return success(response)  # return the value of the success key
-        else:
-            return response
+        response = success(response)
+        if isinstance(response, i3Exception):
+            raise response
+        return response
     function.__name__ = type
     function.__doc__ = 'Message sender (type: %s, message: %s)' % (type, message)
     return function
@@ -425,7 +418,7 @@ def __function__(type, message=''):
 
 def subscribe(event_type, event=None, callback=None):
     """
-    Excepts an event_type and event itself.
+    Accepts an event_type and event itself.
     Creates a new subscription, prints data on every event until
     KeyboardInterrupt is raised.
     """
@@ -448,7 +441,7 @@ def subscribe(event_type, event=None, callback=None):
 
 def get_socket_path():
     """
-    Get the path via i3 command.
+    Gets the socket path via i3 command.
     """
     cmd = ['i3', '--get-socketpath']
     output = __call_cmd__(cmd)
@@ -457,43 +450,80 @@ def get_socket_path():
 
 def success(response):
     """
-    Convenience method for checking success value.
-    Returns None if the "success" key isn't in the received message.
+    Convenience method for filtering success values of a response.
+    Each success dictionary is replaces with boolean value.
+    i3.MessageError is returned if error key is found in any of the
+    success dictionaries.
     """
     if isinstance(response, dict) and 'success' in response:
+        if 'error' in response:
+            return MessageError(response['error'])
         return response['success']
+    elif isinstance(response, list):
+        for index, item in enumerate(response):
+            item = success(item)
+            if isinstance(item, i3Exception):
+                return item
+            response[index] = item
     return response
 
 
 def container(**criteria):
     """
-    Turns keyword arguments into a formatted contaner criteria.
+    Turns keyword arguments into a formatted container criteria.
     """
     criteria = ['%s="%s"' % (key, val) for key, val in criteria.items()]
     return '[%s]' % ' '.join(criteria)
 
 
-def filter(tree=None, **conditions):
+def parent(con_id, tree=None):
+    """
+    Searches for a parent of a node/container, given the container id.
+    Returns None if no container with given id exists (or if the
+    container is already a root node).
+    """
+    def has_child(node):
+        for child in node['nodes']:
+            if child['id'] == con_id:
+                return True
+        return False
+    parents = filter(tree, has_child)
+    if not parents or len(parents) > 1:
+        return None
+    return parents[0]
+
+ 
+def filter(tree=None, function=None, **conditions):
     """
     Filters a tree based on given conditions. For example, to get a list of
     unfocused windows (leaf nodes) in the current tree:
       i3.filter(nodes=[], focused=False)
+    The return value is always a list of matched items, even if there's
+    only one item that matches.
+    The user function should take a single node. The function doesn't have
+    to do any dict key or index checking (this is handled by i3.filter
+    internally).
     """
-    if tree == None:
+    if tree is None:
         tree = msg('get_tree')
     elif isinstance(tree, list):
         tree = {'nodes': tree}
-    if not conditions:
-        conditions = {}
-    for key, value in conditions.items():
-        if key not in tree or tree[key] != value:
-            break
+    if function:
+        try:
+            if function(tree):
+                return [tree]
+        except (KeyError, IndexError):
+            pass
     else:
-        return [tree]
+        for key, value in conditions.items():
+            if key not in tree or tree[key] != value:
+                break
+        else:
+            return [tree]
     matches = []
     if 'nodes' in tree:
         for node in tree['nodes']:
-            matches.extend(filter(node, **conditions))
+            matches.extend(filter(node, function, **conditions))
     return matches
 
 
